SystemOrganization addCategory: #'Xpf-RestSupport'!

RestPrefetchProxyCollection subclass: #XpfProxyCollection
	instanceVariableNames: 'updateCounter'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Xpf-RestSupport'!

!XpfProxyCollection methodsFor: 'adding' stamp: 'mt 5/7/2009 10:54'!
fill: anObject

	super fill: anObject.
	self updateCounter: self updateInterval.! !

!XpfProxyCollection methodsFor: 'updating' stamp: 'mt 5/7/2009 10:51'!
needsUpdate

	| needsUpdate |
	self updateCounter: self updateCounter - 1.
	needsUpdate := self updateCounter = 0.
	self updateCounter <= 0 ifTrue: [self updateCounter: self updateInterval].
	^ needsUpdate! !

!XpfProxyCollection methodsFor: 'accessing' stamp: 'cs 6/11/2009 14:53'!
revision

	| shortRestUri |
	self isLoaded ifFalse: [^ nil].
	shortRestUri := String new.
	(self restUri findTokens: $/) allButLast
		do: [:each | shortRestUri := shortRestUri, each]
		separatedBy: [shortRestUri := shortRestUri, '/'].
	shortRestUri ifEmpty: [^ 0].
	"I am a collection with no parent therefore I have no revision. 0 means always update"
	^ (self repository getObject: '/', shortRestUri) revision! !

!XpfProxyCollection methodsFor: 'accessing' stamp: 'mt 5/7/2009 10:53'!
updateCounter

	^ updateCounter ifNil: [0]! !

!XpfProxyCollection methodsFor: 'accessing' stamp: 'mt 5/7/2009 10:53'!
updateCounter: anObject

	updateCounter := anObject.! !

!XpfProxyCollection methodsFor: 'updating' stamp: 'cs 6/11/2009 14:55'!
updateInterval
	"Tell the client how often a changes request for this entity should be sent."
	
	| shortRestUri parent |
	self isLoaded ifFalse: [^ nil].
	shortRestUri := String new.
	(self restUri findTokens: $/) allButLast
		do: [:each | shortRestUri := shortRestUri, each]
		separatedBy: [shortRestUri := shortRestUri, '/'].
	shortRestUri ifEmpty: [^ 0].
	"I am a collection with no parent therefore I have no revision. 0 means never update"
	parent := self repository getObject: '/', shortRestUri.
	^ parent isLoaded ifTrue: [parent updateInterval] ifFalse: [0]! !

RestPrefetchProxyObject subclass: #XpfProxyObject
	instanceVariableNames: 'updateCounter'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Xpf-RestSupport'!

!XpfProxyObject methodsFor: 'accessing' stamp: 'mt 4/27/2009 14:47'!
currentUser

	^ self repository currentUser! !

!XpfProxyObject methodsFor: 'updating' stamp: 'cs 5/15/2009 15:04'!
needsUpdate

	| needsUpdate |
	self isDeleted ifTrue: [^ false].
	self updateCounter: self updateCounter - 1.
	needsUpdate := self updateCounter = 0.
	self updateCounter <= 0 ifTrue: [self updateCounter: self updateInterval].
	^ needsUpdate! !

!XpfProxyObject methodsFor: 'helping' stamp: 'cs 4/17/2009 16:52'!
parseJsonKey: key value: value

	^ key = 'timestamp'
		ifTrue: [key -> (TimeStamp fromString: value)]
		ifFalse: [super parseJsonKey: key value: value].! !

!XpfProxyObject methodsFor: 'printing' stamp: 'cs 4/17/2009 13:11'!
printString

	(self includesProperty: #title) ifTrue: [^ self title].
	(self includesProperty: #userName) ifTrue: [^ self userName].
	^ super printString! !

!XpfProxyObject methodsFor: 'accessing' stamp: 'cs 6/11/2009 14:55'!
properties: aDictionary 

	| wasLoaded |
	wasLoaded := self isLoaded and: [self isDeleted not].
	super properties: aDictionary.
	self updateCounter: self updateInterval.
	(wasLoaded and: [self isDeleted]) ifTrue: [self delete].! !

!XpfProxyObject methodsFor: 'accessing' stamp: 'mig 4/3/2009 15:29'!
revision

	^ properties isNil
		ifTrue: [nil]
		ifFalse: [super revision]! !

!XpfProxyObject methodsFor: 'accessing' stamp: 'mt 5/7/2009 10:38'!
updateCounter

	^ updateCounter ifNil: [0]! !

!XpfProxyObject methodsFor: 'accessing' stamp: 'mt 5/7/2009 10:34'!
updateCounter: anObject

	updateCounter := anObject.! !

Object subclass: #XpfLoginProcessor
	instanceVariableNames: 'connectionString repository'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Xpf-RestSupport'!

!XpfLoginProcessor class methodsFor: 'class initialization' stamp: 'mt 12/30/2009 11:09'!
initialize

	super initialize.
	Preferences addTextPreference: #xpfServer category: #xpf default: 'http://xpforums2.hpi.uni-potsdam.de:8082/' balloonHelp: 'Server name or address for ProjectTalk'.! !

!XpfLoginProcessor class methodsFor: 'accessing' stamp: 'topa 9/30/2009 14:04'!
serverString

	^ Preferences valueOfPreference: #xpfServer! !

!XpfLoginProcessor methodsFor: 'accessing' stamp: 'topa 10/29/2009 11:54'!
connectionString

	ActiveHand ensureLoginAndPasswordExistAndDo: [:login :password | |url|
		url := self class serverString asUrl.
		^ 'http://{1}:{2}@{3}:{4}/{5}' format: {login encodeForHTTP. password. url authority. url port. url fullPath}].
	^ self class serverString! !

!XpfLoginProcessor methodsFor: 'accessing' stamp: 'mt 6/11/2009 14:23'!
project

	| projectChooser selectedProject |
	self user projects isEmpty ifTrue: [
		UITheme current alertIn: World text: 'You are not a member of any project.' title: 'Error'.
		XpfClientOpenError signal].
	selectedProject := (self user projects size = 1)
		ifTrue: [selectedProject := self user projects first]
		ifFalse: [
			projectChooser := XpfProjectChooser new
			passwordChangeDialog: XpfChangePasswordDialog new.		
			projectChooser passwordChangeDialog model user: self user.
			projectChooser model projects: self user projects.	
			(World openModal: projectChooser) result ].
		
	^ selectedProject ifNil: [XpfClientOpenError signal]! !

!XpfLoginProcessor methodsFor: 'accessing' stamp: 'mt 12/30/2009 15:25'!
projectForHand: aHand

	^ aHand providesProject
		ifTrue: [aHand project]
		ifFalse: [self project]
! !

!XpfLoginProcessor methodsFor: 'accessing' stamp: 'cs 5/26/2009 16:42'!
repository

	repository ifNil: [
		self repository: (XpfProxyRepository fromString: self connectionString)].
	^ repository! !

!XpfLoginProcessor methodsFor: 'accessing' stamp: 'cs 5/26/2009 16:41'!
repository: anObject

	repository := anObject.! !

!XpfLoginProcessor methodsFor: 'reseting' stamp: 'SR 7/22/2009 11:19'!
reset

	self
		repository: nil.! !

!XpfLoginProcessor methodsFor: 'private' stamp: 'topa 9/30/2009 14:27'!
user

	| userResponse |
	userResponse := [RestSocket get: self connectionString , '/info']
		on: RestConnectionLostError
		do: [UITheme current
				alertIn: World
				text: 'Could not connect to the server.'
				title: 'Connection failed'.
			XpfClientOpenError signal].
	userResponse isOk ifFalse: [
		userResponse isUnauthorized ifTrue: [
			UITheme current
				alertIn: World
				text: 'You are not authorized to access any resource on the server.'
				title: 'Connection failed'.
			XpfClientOpenError signal]
			ifFalse: [RestProxyRepository errorForCode: userResponse responseCode]].
	^ (self repository getObject: userResponse content)! !

RestPrefetchProxyRepository subclass: #XpfProxyRepository
	instanceVariableNames: 'updateProcess updateCounter currentUser'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Xpf-RestSupport'!

!XpfProxyRepository class methodsFor: 'constants' stamp: 'mig 4/3/2009 15:32'!
proxyCollectionClass

	^ XpfProxyCollection! !

!XpfProxyRepository class methodsFor: 'constants' stamp: 'mig 4/3/2009 15:26'!
proxyObjectClass

	^ XpfProxyObject! !

!XpfProxyRepository class methodsFor: 'constants' stamp: 'mig 4/3/2009 10:56'!
reloadInterval
	"reload interval in seconds"
	
	^ 10! !

!XpfProxyRepository methodsFor: 'add-remove' stamp: 'cs 6/3/2009 15:33'!
addProject: aProject 

	^ self projects addNew: aProject! !

!XpfProxyRepository methodsFor: 'add-remove' stamp: 'cs 6/3/2009 15:51'!
addUser: aUser 

	^ self users addNew: aUser! !

!XpfProxyRepository methodsFor: 'updating' stamp: 'mig 4/1/2009 16:51'!
changesUrl
	
	^ (self url asString, '/changes') asUrl! !

!XpfProxyRepository methodsFor: 'accessing' stamp: 'cs 5/26/2009 16:40'!
currentUser

	^ currentUser! !

!XpfProxyRepository methodsFor: 'accessing' stamp: 'mt 4/27/2009 14:37'!
currentUser: anXpfUser
	
	currentUser := anXpfUser.! !

!XpfProxyRepository methodsFor: 'initialization' stamp: 'cs 5/28/2009 12:45'!
initialize

	super initialize.
	self updateCounter: 0.! !

!XpfProxyRepository methodsFor: 'accessing' stamp: 'cs 6/3/2009 15:32'!
projects

	^ self getCollection: '/projects/'! !

!XpfProxyRepository methodsFor: 'private' stamp: 'mig 4/3/2009 10:55'!
pvtUpdateProcess

	^ updateProcess! !

!XpfProxyRepository methodsFor: 'private' stamp: 'mig 4/3/2009 10:54'!
pvtUpdateProcess: anObject

	updateProcess := anObject.! !

!XpfProxyRepository methodsFor: 'updating' stamp: 'SR 7/22/2009 22:14'!
reload
	"Sends the URIs of all loaded proxies to the server and retrieves the updates"

	| response |
	response := RestProxyObject socketClass postDocumentFromUrl: self changesUrl request: '' content: 
		(Json render: (Dictionary newFrom:
			((self cache associations
				reject: [:assoc | assoc value revision isNil])
				select: [:assoc | assoc value needsUpdate]
				thenCollect: [:assoc | assoc key -> assoc value revision]))).
	response isOk
		ifTrue: [
			(Json readFrom: (response content readStream))
				do: [:uri |
					(self cache includesKey: uri) "Check this because the cache will be modified during the iteration."
						ifTrue: [(self cache at: uri) reloadNeeded]]]
		ifFalse: [self inform: response content].! !

!XpfProxyRepository methodsFor: 'initialization' stamp: 'cs 5/28/2009 12:46'!
startUpdating
	
	self pvtUpdateProcess ifNil: [self pvtUpdateProcess: self updater fork].
	self updateCounter: self updateCounter + 1.! !

!XpfProxyRepository methodsFor: 'updating' stamp: 'mt 12/8/2009 15:53'!
stopUpdating

	self updateCounter: self updateCounter - 1.
	(self updateCounter <= 0 & self pvtUpdateProcess notNil) ifTrue: [
		self pvtUpdateProcess terminate.
		self pvtUpdateProcess: nil.
		self changed: #reloadStopped].! !

!XpfProxyRepository methodsFor: 'accessing' stamp: 'mt 6/2/2009 14:43'!
updateCounter

	^ updateCounter
		ifNil: [updateCounter := 0]! !

!XpfProxyRepository methodsFor: 'accessing' stamp: 'cs 5/28/2009 12:45'!
updateCounter: anObject

	updateCounter := anObject.! !

!XpfProxyRepository methodsFor: 'updating' stamp: 'mt 12/8/2009 16:27'!
updater

	^ [
		[
			self reload.
			self changed: #nextReload with: self class reloadInterval.
			(Delay forSeconds: self class reloadInterval) wait
		] repeat
	]! !

!XpfProxyRepository methodsFor: 'accessing' stamp: 'cs 6/3/2009 15:32'!
users

	^ self getCollection: '/users/'! !

Error subclass: #XpfClientOpenError
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Xpf-RestSupport'!
XpfLoginProcessor initialize!
